from abc import ABCMeta, abstractmethod


class Constraint(metaclass=ABCMeta):

    # Public

    @abstractmethod
    def __call__(self, emitter):
        pass  # pragma: no cover

    def __bool__(self):
        return True

    def extend(self, name, value):
        pass


class CompositeConstraint(Constraint, metaclass=ABCMeta):

    # Public

    def __init__(self):
        self.__include = []
        self.__exclude = []

    def __bool__(self):
        return bool(self._include or self._exclude)

    def __repr__(self):
        return ('Include: {self._include}, exclude: {self._exclude}'.
                format(self=self))

    # Protected

    @property
    def _include(self):
        return self.__include

    @property
    def _exclude(self):
        return self.__exclude


class PatternConstraint(CompositeConstraint, metaclass=ABCMeta):

    # Public

    def __call__(self, emitter):
        for pattern in self._include:
            if not self._match(emitter, pattern):
                emitter.skip()
        for pattern in self._exclude:
            if self._match(emitter, pattern):
                emitter.skip()

    # Protected

    @abstractmethod
    def _match(self, emitter, pattern):
        pass  # pragma: no cover
