import re
import unittest
from unittest.mock import Mock
from importlib import import_module
component = import_module('run_find.objects.objname')


class ObjnameConstraintTest(unittest.TestCase):

    # Actions

    def setUp(self):
        self.constraint = component.ObjnameConstraint()
        self.emitter = Mock(objname='objname')

    # Tests

    def test___call___with_objname(self):
        self.constraint.extend('objname', 'objname')
        self.constraint(self.emitter)
        self.assertFalse(self.emitter.skip.called)

    def test___call___with_objname_skip(self):
        self.constraint.extend('objname', 'x')
        self.constraint(self.emitter)
        self.assertTrue(self.emitter.skip.called)

    def test___call___with_objname_is_regex(self):
        self.constraint.extend('objname', re.compile('o.*'))
        self.constraint(self.emitter)
        self.assertFalse(self.emitter.skip.called)

    def test___call___with_objname_is_regex_skip(self):
        self.constraint.extend('objname', re.compile('x.*'))
        self.constraint(self.emitter)
        self.assertTrue(self.emitter.skip.called)

    def test___call___with_notobjname(self):
        self.constraint.extend('notobjname', 'x')
        self.constraint(self.emitter)
        self.assertFalse(self.emitter.skip.called)

    def test___call___with_notobjname_skip(self):
        self.constraint.extend('notobjname', 'objname')
        self.constraint(self.emitter)
        self.assertTrue(self.emitter.skip.called)
